/*
 Nokia 3120 KSS (Keypad SMS Sender)
 Version: 1.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 26/12/2011
 Last Modified: 29/01/2012
 
 Uses an ATtiny261 with 1x ULN2803A and 1xULN2003A to send an SMS using the Nokia 3120 keypad.
 
 */
 
// ATtiny261 Pin map
//
//               +-\/-+
//         PB0   1|o   |20  PA0 (ADC0)
//         PB1   2|    |19  PA1 (ADC1)
//         PB2   3|    |18  PA2 (ADC2)
//         PB3   4|    |17  PA3
//         VCC   5|    |16  AGND
//         GND   6|    |15  AVCC
//  (ADC7) PB4   7|    |14  PA4 (ADC3)
//  (ADC8) PB5   8|    |13  PA5 (ADC4)
//  (ADC9) PB6   9|    |12  PA6 (ADC5)
//   Reset PB7  10|    |11  PA7 (ADC6)
//               +----+
//
//
// Nokia 3120 Button map
// PB4 - Nokia power button
//
//					PB0		PB1		PB2		PB3		PB3
//					1			3			4			7			6
// PA0	2		Select	Up			Exit		Down
// PA1	5					Dial					Hangup
// PA2	9		But2					But0		But8
// PA3	10		But3		But6		Hash		But9
// PA4	8		But1		But4		Star		But7
// PA5	11														Left
// PA6	12														Right
// PA7	13														But5

#define F_CPU 1000000 // 1 MHz clock

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "setup.c"

prog_char smsMessage[] PROGMEM = "hi this is a test sms which is typed using the keypad. by insidegadgets.";
prog_char mobileNumber[] PROGMEM = "0412345678";

// Break up the button number, turn on the necessary outputs, delay and then turn off
void pushButton(char buttonNumber) {
	int outputPBon = (buttonNumber % 10) - 1;
	int outputPAon = (buttonNumber / 10) - 1;
	
	PORTB |= (1<<outputPBon);
	PORTA = (1<<outputPAon);
	watchdogSleep(T128MS);
	
	PORTA = 0;
	if (outputPBon >= 0) { 
		PORTB &= ~(1<<outputPBon);
	}
	watchdogSleep(T128MS);
}

// Find which button to press for the character received
void parseChar(char c, byte onePushnumbers) {
	
	// Convert lowercase a-z to uppercase
	if (c >= 97 && c <= 122) {
		c -= 32;
	}
	
	// A-Z charset and 0-9 charset
	if ((c >= 65 && c <= 90) || (c >= 48 && c <= 57)) {
		byte lookupChar;
		byte loopEnd;
		byte buttonPressing;
		
		if (c >= 65 && c <= 90) {// A-Z charset
			lookupChar = c - 65;
			loopEnd = letterTobutton[lookupChar][buttonPushes];
			buttonPressing = letterTobutton[lookupChar][buttonTopush];
		}
		else if (c >= 48 && c <= 57) { //0-9 charset
			lookupChar = c - 48;
			loopEnd = numberTobutton[lookupChar][buttonPushes];
			buttonPressing = numberTobutton[lookupChar][buttonTopush];
		}
		
		// If we are in a section that allows numbers to be pressed just once (like dialing a number)
		if (onePushnumbers == true) {
			pushButton(buttonPressing);
		}
		else {
			for (int x = 0; x < loopEnd; x++) { // Push the corresponding button x times
				pushButton(buttonPressing);
			}
		}
	}
	
	// Dot
	else if (c == 46) {
		pushButton(button1);
	}
	
	// Space
	else if (c == 32) {
		pushButton(button0);
	}
}

// Send SMS message
void sendSMS(char *smsMsg, char *phoneNumber) {
	
	// Exit out anything that might be up
	pushButton(buttonHangup);
	watchdogSleep(T500MS);
	
	// Create new SMS message
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	watchdogSleep(T500MS);
	
	// Parse message
	int charAt = 0;
	char c;
	while ((c = (char)pgm_read_byte(&smsMsg[charAt]))) {
		parseChar(c, 0);
		charAt++;
		pushButton(buttonRight);
	}
	
	// Enter phone number
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	watchdogSleep(T500MS);
	
	// Parse phone number
	charAt = 0;
	while ((c = (char)pgm_read_byte(&phoneNumber[charAt]))) {
		parseChar(c, numbersPushedonce);
		charAt++;
	}
	
	// Send
	pushButton(buttonSelect);
	
	// Wait a few seconds for message to be sent
	watchdogSleep(T4S);
}

// Power on/off Nokia phone
void powerPress(int button) {
	PORTB |= (1<<buttonPower);
	watchdogSleep(T2S);
	PORTB &= ~(1<<buttonPower);
	if (button == ON) { // Phone turned on, wait a few seconds until phone is ready
		watchdogSleep(T8S);
	} 
}

int main(void) {
	
	setup();
	
	// Set Outputs
	DDRA = 0xFF;
	DDRB |= ((1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB3) | (1<<PB4));
	
	// Turn on pull-up resistors to save power on un-used ports
	PORTB |= ((1<<PB7) | (1<<PB6) | (1<<PB5));
	
	while(1) {
		powerPress(ON);
		sendSMS(smsMessage, mobileNumber);
		powerPress(OFF);
		system_sleep();
	}
}

ISR(WDT_vect) { }
